import 'ng-wig/dist/ng-wig';

import * as angular from 'angular';
import * as _ from 'lodash';

import { services, downgrade } from 'typescript-angular-utilities';
import __object = services.object;

import { richTextEditorProvider, providerName, IRichTextEditorProvider } from './richTextEditor.config';
import { headerButton, headerButtonDirectiveName, } from './headerButton';
import { paragraphButton, paragraphButtonDirectiveName, } from './paragraphButton';

let externalProviderName: string = providerName + 'Provider';
export { externalProviderName as providerName, IRichTextEditorProvider };

export var moduleName: string = 'rl.ui.components.richTextEditor';
export var componentName: string = 'rlRichTextEditor';
export var controllerName: string = 'RichTextEditorController';

export interface IRichTextEditorBindings {
	ngModel: string;
	customButtons: string[];
	ngDisabled: boolean;
}

export class RichTextEditorController {
	// bindings
	ngModel: string;
	customButtons: string;
	ngDisabled: boolean;

	toolbar: string;

	static $inject: string[] = [downgrade.objectServiceName, providerName];
	constructor(object: __object.IObjectUtility, provider: void) {
		this.toolbar = 'h1, paragraph, bold, italic, underline, list1, list2, indent, outdent';

		if (!object.isNullOrEmpty(this.customButtons)) {
			this.toolbar += ', ' + this.customButtons;
		}
	}
}

let richTextEditor: angular.IComponentOptions = {
	template: require('./richTextEditor.html'),
	controller: controllerName,
	controllerAs: 'editor',
	bindings: {
		ngModel: '=',
		customButtons: '=',
		ngDisabled: '=',
	},
};

angular.module(moduleName, ['ngWig', downgrade.moduleName])
	.component(componentName, richTextEditor)
	.controller(controllerName, RichTextEditorController)
	.directive(headerButtonDirectiveName, headerButton)
	.directive(paragraphButtonDirectiveName, paragraphButton)
	.provider(providerName, richTextEditorProvider);
